//: /be/optiloading/DrawBM.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.ArrayList;

/** 
 * DrawBM class is used to draw the aproximate Bending Moment diagram.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class DrawBM extends JPanel{
	private ArrayList<Integer> x;
	private ArrayList<Integer> y;
	
	/**
	 * Constructor
	 */
	public DrawBM(){
		Dimension d5 = new Dimension(599,300);
		setSize(d5);
		newList();
		addList(584,150);
	}
	
	/**
	 * Overwritten paintComponent method to draw the GZ-curve on the panel.
	 * @param g Graphics object containing what to render.
	 */
	public void paintComponent(Graphics g){
		Graphics2D g2d = (Graphics2D)g;
		
		RenderingHints rh = new RenderingHints(
				RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setRenderingHints(rh);
		
		g2d.setColor(new Color(115,183,253)); //back color
		g2d.fillRect(0,0,599,300);
		
		g2d.setColor(Color.black);
		g2d.drawLine(27,150,594,150); //x-axis
		
		Stroke defaultStroke = g2d.getStroke();
		float[] dashPattern = { 3, 3, 3, 3};
		g2d.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10, dashPattern, 0));
		g2d.drawLine(37,0,37,300); //AP mark
		g2d.drawLine(584,0,584,300); //FP mark
		g2d.drawLine(131,0,131,300); //Frame 46
		g2d.drawLine(203,0,203,300); //Frame 78
		g2d.drawLine(275,0,275,300); //Frame 110
		g2d.drawLine(347,0,347,300); //Frame 142
		g2d.drawLine(419,0,419,300); //Frame 174
		g2d.drawLine(491,0,491,300); //Frame 206
		g2d.drawLine(565,0,565,300); //Frame 239
		g2d.drawLine(27,125,594,125); //50000 mark
		g2d.drawLine(27,100,594,100); // 100000 mark
		g2d.drawLine(27,75,594,75); // 150000 mark
		g2d.drawLine(27,50,594,50); // 200000 mark
		g2d.drawLine(27,25,594,25); // 250000 mark
		g2d.drawLine(27,175,594,175); //-50000 mark
		g2d.drawLine(27,200,594,200); // -100000 mark
		g2d.drawLine(27,225,594,225); // -150000 mark
		g2d.drawLine(27,250,594,250); // -200000 mark
		g2d.drawLine(27,275,594,275); // -250000 mark
		Font font = new Font("Dialog", Font.BOLD, 12);
		g2d.setFont(font);
		g2d.drawString("kton.m",3,14);
		font = new Font("Dialog", Font.PLAIN, 10);
		g2d.setFont(font);
		g2d.drawString("0",14,154);
		g2d.drawString("50",12,129);
		g2d.drawString("100",6,104);
		g2d.drawString("150",6,79);
		g2d.drawString("200",6,54);
		g2d.drawString("250",6,29);
		g2d.drawString("-50",7,179);
		g2d.drawString("-100",1,204);
		g2d.drawString("-150",1,229);
		g2d.drawString("-200",1,254);
		g2d.drawString("-250",1,279);
		font = new Font("Dialog", Font.PLAIN, 10);
		g2d.setFont(font);
		g2d.drawString("AP",23,161);
		g2d.drawString("FP",586,161);
		g2d.drawString("Tank 1",514,296);
		g2d.drawString("Tank 2",439,296);
		g2d.drawString("Tank 3",367,296);
		g2d.drawString("Tank 4",295,296);
		g2d.drawString("Tank 5",223,296);
		g2d.drawString("Tank 6",151,296);
		
		//Still water limit
		g2d.setColor(Color.yellow);
		g2d.setStroke(defaultStroke);
		g2d.drawLine(37,150,92,135);
		g2d.drawLine(92,135,146,74);
		g2d.drawLine(146,74,201,48);
		g2d.drawLine(201,48,420,48);
		g2d.drawLine(420,48,529,135);
		g2d.drawLine(529,135,584,150);
		g2d.drawLine(37,150,92,163);
		g2d.drawLine(92,163,201,236);
		g2d.drawLine(201,236,420,236);
		g2d.drawLine(420,236,529,163);
		g2d.drawLine(529,163,584,150);
		
		//Wave limit
		g2d.setColor(Color.red);
		g2d.drawLine(37,150,92,139);
		g2d.drawLine(92,139,146,93);
		g2d.drawLine(146,93,201,74);
		g2d.drawLine(201,74,420,74);
		g2d.drawLine(420,74,529,139);
		g2d.drawLine(529,139,584,150);
		g2d.drawLine(37,150,92,160);
		g2d.drawLine(92,160,201,215);
		g2d.drawLine(201,215,420,215);
		g2d.drawLine(420,215,529,160);
		g2d.drawLine(529,160,584,150);
		
		//Real values
		g2d.setColor(Color.white);
		GeneralPath graph = new GeneralPath();
		graph.moveTo(37,150);
		//The line is made up from the 2 coordinate lists
		for(int i = 0; i<x.size(); i++){
			graph.lineTo(x.get(i),y.get(i));
		}
		g2d.draw(graph);
	}
	
	/**
	 * Do a BM correction seeing the approximation isn't totally accurate.
	 */
	public void bmCorrection(){
		for(int i=247;i > 235;i--){
			x.remove(i);
			y.remove(i);
		}
		addList(584,150);
	}
	
	/**
	 * Initiate a new list
	 */
	public void newList(){
		x = new ArrayList<Integer>();
		y = new ArrayList<Integer>();
	}
	
	/**
	 * Add x and y coordinates to the list
	 * @param xvalue Integer indicating the x coordinate.
	 * @param yvalue Integer indicating the y coordinate.
	 */
	public void addList(int xvalue, int yvalue){
		x.add(xvalue);
		y.add(yvalue);
	}
}