//: /be/optiloading/DrawGZ.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import javax.swing.*;
import java.awt.*;
import java.math.*;
import java.awt.geom.*;

/** 
 * DrawGZ class is used to draw the aproximate GZ-curve using the Wall-sided formula.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class DrawGZ extends JPanel{
	
	private float gm;
	private float bm;
	
	/**
	 * Constructor to set the dimensions and the initial GM and BM.
	 */
	public DrawGZ(){
		Dimension d5 = new Dimension(585,300);
		setPreferredSize(d5);
		setMinimumSize(d5);
		setMaximumSize(d5);
		gm = 0f;
		bm = 0f;
	}
	
	/**
	 * Overwritten paintComponent method to draw the GZ-curve on the panel.
	 * @param g Graphics object containing what to render.
	 */
	public void paintComponent(Graphics g){
		float t;                      //angle (degree)
		double tt;                  //angle (radian)
		int x1,y1,x2,y2;            //two points
		float y;

		Graphics2D g2d = (Graphics2D)g;
		
		//RenderingHint to properly display the curve
		RenderingHints rh = new RenderingHints(
				RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setRenderingHints(rh);
		
		g2d.setColor(new Color(115,183,253)); //back color
		g2d.fillRect(0,0,585,300);
		
		g2d.setColor(Color.black);    //color of axes
		Font font = new Font("Dialog", Font.PLAIN, 10);
		g2d.setFont(font);
		g2d.drawLine(51,260,575,260);  //x axis
		g2d.drawLine(575,260,571,256); //arrow top
		g2d.drawLine(575,260,571,264); //arrow bottom
		g2d.drawLine(225,256,225,264); // 10 mark
		g2d.drawLine(395,256,395,264); // 20 mark
		g2d.drawLine(565,256,565,264); // 30 mark
		g2d.drawString("0",52,275);
		g2d.drawString("10",219,275);
		g2d.drawString("20",389,275);
		g2d.drawString("30",558,275);
		
		g2d.drawLine(55,10,55,265);  //y axis
		g2d.drawLine(55,10,51,14); //arrow left
		g2d.drawLine(55,10,59,14); //arrow right
		g2d.drawLine(51,230,59,230); // 1 mark
		g2d.drawLine(51,200,59,200); // 2 mark
		g2d.drawLine(51,170,59,170); // 3 mark
		g2d.drawLine(51,140,59,140); // 4 mark
		g2d.drawLine(51,110,59,110); // 5 mark
		g2d.drawLine(51,80,59,80); // 6 mark
		g2d.drawLine(51,50,59,50); // 7 mark
		g2d.drawLine(51,20,59,20); // 8 mark
		g2d.drawString("1",40,236);
		g2d.drawString("2",40,206);
		g2d.drawString("3",40,176);
		g2d.drawString("4",40,146);
		g2d.drawString("5",40,116);
		g2d.drawString("6",40,86);
		g2d.drawString("7",40,56);
		g2d.drawString("8",40,26);

		g2d.setColor(Color.white);     //color of the graph
		for(int i=55;i<565;i++){
			t = (float) (i-55)/17;
			tt=(double) (t)/180*Math.PI; //degree to radian    
			x1=(int)(i);// point 1
			y = (float) (Math.sin(tt)*(gm + 0.5*bm*Math.tan(tt)*Math.tan(tt))); //Calculation with the wall-sided formula
			y1=(int)(-30*y+260);

			t = (float) (i-54)/17;
			tt=(double) (t)/180*Math.PI;
			x2=(int)(i+1);// point 2
			y = (float) (Math.sin(tt)*(gm + 0.5*bm*Math.tan(tt)*Math.tan(tt))); //Calculation with the wall-sided formula
			y2=(int)(-30*y+260);
			g2d.drawLine(x1,y1,x2,y2);    //connect 2 points
		}
		
		//Legenda
		g2d.setColor(Color.black); 
		font = new Font("Dialog", Font.PLAIN, 14);
		g2d.setFont(font);
		g2d.drawString("Angle in degrees",267,285);
		g2d.rotate(-Math.PI/2, 25, 192);
		g2d.drawString("GZ in meters", 25, 192);
	}
	
	/**
	 * Method to set the GM and BM which is used to calculate the GZ.
	 * @param gme Floating point value indicating the GM.
	 * @param bme Floating point value indicating the BM.
	 */
	public void setValues(float gme, float bme){
		gm = gme;
		bm = bme;
	}
}