//: /be/optiloading/DrawShip.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import javax.swing.*;
import java.awt.*;
import java.math.*;
import java.awt.geom.*;
import java.util.ArrayList;
import java.net.*;
import java.awt.image.*;

/** 
 * DrawShip class which creates a panel with the ship and all rectangles which display the tank levels.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class DrawShip extends JPanel{
	
	private ArrayList<Float> volumes;
	/**
	 * Constructor which draws the ship
	 */
	public DrawShip(){
		//Load and add image
		URL imageURL = this.getClass().getResource("/images/danita.png");
		ImageIcon danita = new ImageIcon(imageURL);
		JLabel image = new JLabel(danita);
		add(image);
		
		//Initilaize the volumes ArrayList
		volumes = new ArrayList<Float>();
		for(int i = 0; i < 6;i++){
			volumes.add(0f);
		}
	}
	
	/**
	 * Overwritten paintComponent method to draw the rectangles which display the tank levels.
	 * @param g Graphics object containing what to render.
	 */
	public void paintComponent(Graphics g){
		int[] tanklevels = new int[6]; // Stow the tank levels in percentage in an array of integer values.
		Ship loadedship = Ship.getInstance(); 
		for(int i = 0; i<6;i++){ // Run trough the tanks and set the levels
			Tank x = loadedship.tanks.get(i);
			float level = volumes.get(i)/x.getMaxvolume();
			tanklevels[i] = Math.round(44 * level);
		}
		loadedship = null;
		
		//Set renderinghints to display the ship at good quality
		Graphics2D g2 = (Graphics2D) g;
		RenderingHints rh = new RenderingHints(
				RenderingHints.KEY_INTERPOLATION,
				RenderingHints.VALUE_INTERPOLATION_BICUBIC);
		rh.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		rh.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY); 
		g2.setRenderingHints(rh);
	
		// Draw every rectangle with at the right place and displaying the correct level in the tank
		g2.setPaint(new Color(71,53,192));
		g2.fill(new Rectangle2D.Double(110,60+(44-tanklevels[5]),75,tanklevels[5]));
		g2.setPaint(new Color(6,207,225));
		g2.fill(new Rectangle2D.Double(185,60+(44-tanklevels[4]),72,tanklevels[4]));
		g2.setPaint(new Color(36,118,166));
		g2.fill(new Rectangle2D.Double(258,60+(44-tanklevels[3]),72,tanklevels[3]));
		g2.setPaint(new Color(29,187,144));
		g2.fill(new Rectangle2D.Double(331,60+(44-tanklevels[2]),72,tanklevels[2]));
		g2.setPaint(new Color(7,104,75));
		g2.fill(new Rectangle2D.Double(404,60+(44-tanklevels[1]),73,tanklevels[1]));
		g2.setPaint(new Color(48,217,72));
		g2.fill(new Rectangle2D.Double(478,60+(44-tanklevels[0]),74,tanklevels[0]));
	}
	
	/**
	 * Method to set the volumes which have to be displayed.
	 * @param vol ArrayList of floating point values indicating the volumes to be displayed.
	 */
	public void setVolumes(ArrayList<Float> vol){
		volumes = vol;
	}
}