//: be/optiloading/HelpViewer.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.text.html.*;
import java.net.*;
import java.io.IOException;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

/** 
 * HelpViewer class is used to create the viewer for the help information.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class HelpViewer extends JFrame{
	private JEditorPane htmlPane;
	private JTree tree;
	private DefaultTreeModel model;
	private JButton back;
	private JButton forward;
	private ArrayList<URL> history = new ArrayList<URL>(); //Holds the previous viewed pages to enabled the back/forward buttons
	private int backvalue = 0; //Number indicating how many places in the history the user went back
	private boolean hyperlink = false;
	
	public HelpViewer(){
		//Create main panel
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		
		//Create top node and add the child nodes
		DefaultMutableTreeNode top = new DefaultMutableTreeNode(new HelpInfo("OptiLoading User's Guide","/help/welcome.html"));
		createNodes(top);
		
		//Create the tree
		tree = new JTree(top);
        tree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.addTreeSelectionListener(new TreeListener());
		DefaultTreeCellRenderer renderer = new
			DefaultTreeCellRenderer();
		renderer.setOpenIcon(new ImageIcon(HelpViewer.class.getResource("/images/opened.gif")));
		renderer.setClosedIcon(new ImageIcon(HelpViewer.class.getResource("/images/closed.gif")));
		renderer.setLeafIcon(new ImageIcon(HelpViewer.class.getResource("/images/leaf.gif")));
		tree.setCellRenderer(renderer);
		model = (DefaultTreeModel) tree.getModel();
		
		//Create the scroll pane and add the tree to it
        JScrollPane treeView = new JScrollPane(tree);

        //Create the HTML viewing pane
        htmlPane = new JEditorPane();
        htmlPane.setEditable(false);
		htmlPane.addHyperlinkListener(new Hyperactive());
        JScrollPane htmlView = new JScrollPane(htmlPane);

        //Add the scroll panes to a split pane
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
												treeView, htmlView);
		splitPane.setOneTouchExpandable(true);

        htmlView.setPreferredSize(new Dimension(350, 500));
        treeView.setPreferredSize(new Dimension(200, 500));
        splitPane.setDividerLocation(200);
        splitPane.setPreferredSize(new Dimension(550, 500));

        //Add the split pane to the panel
        panel.add(BorderLayout.CENTER, splitPane);
		
		//Create the toolbar
		JToolBar toolbar = new JToolBar();
		toolbar.setFloatable(false);
		toolbar.setRollover(true);
		back = new JButton(new ImageIcon(HelpViewer.class.getResource("/images/Back.gif")));
		back.setToolTipText("Go to previous page");
		forward = new JButton(new ImageIcon(HelpViewer.class.getResource("/images/Forward.gif")));
		forward.setToolTipText("Go to following page");
		JButton home = new JButton(new ImageIcon(HelpViewer.class.getResource("/images/Home.gif")));
		home.setToolTipText("Go home");
		back.setEnabled(false);
		back.addActionListener(new BackListener());
		forward.setEnabled(false);
		forward.addActionListener(new ForwardListener());
		home.addActionListener(new HomeListener());
		toolbar.add(back);
		toolbar.add(forward);
		toolbar.add(home);
		panel.add(BorderLayout.PAGE_START,toolbar); 
		
		//Configure the frame
		setTitle("OptiLoading Help");
		home.doClick();
		getContentPane().add(panel);
		pack();
		setLocationRelativeTo(null);
		setVisible(true);
	}
	
	//ActionListener for the back button
	private class BackListener implements ActionListener{
		public void actionPerformed(ActionEvent e){
			backvalue++;
			displayURL(history.size()-(1+backvalue));
			hyperlink = true;
			searchAndExpand(history.get(history.size()-(1+backvalue)));
			hyperlink = false;
		}
	}
	
	//ActionLitener for the forward
	private class ForwardListener implements ActionListener{
		public void actionPerformed(ActionEvent e){
			backvalue--;
			displayURL(history.size()-(1+backvalue));
			hyperlink = true;
			searchAndExpand(history.get(history.size()-(1+backvalue)));
			hyperlink = false;
		}
	}
	
	//ActionListener for the home button
	private class HomeListener implements ActionListener{
		public void actionPerformed(ActionEvent e){
			history.add(HelpViewer.class.getResource("/help/preface.html"));
			displayURL(history.size()-1);
			hyperlink = true;
			searchAndExpand(HelpViewer.class.getResource("/help/preface.html"));
			hyperlink = false;
		}
	}
	
	//Inner class holding the nodes object
	private class HelpInfo {
        public String helpTitle;
        public URL helpURL;
		
		//Constructor
        public HelpInfo(String title, String filename) {
            helpTitle = title;
            helpURL = HelpViewer.class.getResource(filename);
        }

		//Converts object to string to display the title in the tree
		public String toString() {
	    	return helpTitle;
		}
    }

	//TreeSelectionListener to listen to changes in tree selection
	private class TreeListener implements TreeSelectionListener{
		public void valueChanged(TreeSelectionEvent e) {
	        
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)
	                           tree.getLastSelectedPathComponent();
			
			if (node == null) return;

	       	Object nodeInfo = node.getUserObject();
			HelpInfo help = (HelpInfo)nodeInfo;	
			
			//Hyperlink is true when the search and expand is called (URL should only be added once to history)
			if(hyperlink==false){
				if(backvalue!=0){
					for(int i = 0; i < backvalue; i++){
						history.remove(history.size()-1);
					}
					backvalue = 0;
				}
				history.add(help.helpURL);
				displayURL(history.size()-1);
			}
	    }
	}
	
	//HyperlinkListener listens to hyperlinks from htmlPane
	class Hyperactive implements HyperlinkListener {

	         public void hyperlinkUpdate(HyperlinkEvent e) {
	             if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
	                 JEditorPane pane = (JEditorPane) e.getSource();
	                 if (e instanceof HTMLFrameHyperlinkEvent) {
	                     HTMLFrameHyperlinkEvent  evt = (HTMLFrameHyperlinkEvent)e;
	                     HTMLDocument doc = (HTMLDocument)pane.getDocument();
	                     doc.processHTMLFrameHyperlinkEvent(evt);
	                 } else {
	                     try {
							if(backvalue!=0){
								for(int i = 0; i < backvalue; i++){
									history.remove(history.size()-1);
								}
								backvalue = 0;
							}
							history.add(e.getURL());
	                        displayURL(history.size()-1);
							hyperlink = true;
							searchAndExpand(e.getURL());
							hyperlink = false;
	                     } catch (Throwable t) {
	                         t.printStackTrace();
	                     }
	                 }
	             }
	         }
	     }
	
	//Looks for the node depending on the given URL
	private void searchAndExpand (URL helpURL){
		TreeNode[] path = search((DefaultMutableTreeNode) model.getRoot(), helpURL);
		
		if (path != null) {
			TreePath treePath = new TreePath (path);
			tree.scrollPathToVisible (treePath);
			tree.setSelectionPath (treePath);
			tree.expandPath(treePath);
        }
	}

	//Search all nodes
   private TreeNode[] search (DefaultMutableTreeNode node, URL helpURL) {
		TreeNode[] path = null;
		
		Object nodeInfo = node.getUserObject();
		HelpInfo help = (HelpInfo)nodeInfo;
		
		if (helpURL.equals(help.helpURL)) {
        	path = model.getPathToRoot (node);
        } else {
            int i = 0;
            int n = model.getChildCount(node);

            while ((i < n) && (path == null)) {
            	path = search ((DefaultMutableTreeNode) model.getChild (node, i), helpURL);
 				i ++;
            }
        }
		
		return path;
	}
	
	//Display the URL in the htmlPane
	private void displayURL(int i) {
        try {
        	htmlPane.setPage(history.get(i));
			updateButtons();      
			
        } catch (IOException e) {
			System.err.println("Attempted to read a bad URL");
        }
    }
	
	//Update the back/forward buttons
	public void updateButtons(){
		if(backvalue>0){
			forward.setEnabled(true);
		}else{
			forward.setEnabled(false);
		}
		if((history.size()>1)&&(backvalue<(history.size()-1))){
			back.setEnabled(true);
		} else{
			back.setEnabled(false);
		}
	}
	
	//Create the child nodes
	private void createNodes(DefaultMutableTreeNode top){
		DefaultMutableTreeNode category = null;
        DefaultMutableTreeNode topic = null;

		topic = new DefaultMutableTreeNode(new HelpInfo("Preface","/help/preface.html"));
		top.add(topic);
		
		category = new DefaultMutableTreeNode(new HelpInfo("Release information","/help/release.html"));
		top.add(category);
		topic = new DefaultMutableTreeNode(new HelpInfo("New features","/help/releasenew.html"));
		category.add(topic);
		topic = new DefaultMutableTreeNode(new HelpInfo("Release history","/help/releasehistory.html"));
		category.add(topic);
		topic = new DefaultMutableTreeNode(new HelpInfo("Contents of this release","/help/releasecontents.html"));
		category.add(topic);
		topic = new DefaultMutableTreeNode(new HelpInfo("Requirements","/help/releaserequirements.html"));
		category.add(topic);
		
		category = new DefaultMutableTreeNode(new HelpInfo("General overview","/help/overview.html"));
		top.add(category);
		topic = new DefaultMutableTreeNode(new HelpInfo("Introduction","/help/overviewintroduction.html"));
		category.add(topic);
		topic = new DefaultMutableTreeNode(new HelpInfo("Features","/help/overviewfeatures.html"));
		category.add(topic);
		
		category = new DefaultMutableTreeNode(new HelpInfo("Input information","/help/input.html"));
		top.add(category);
		topic = new DefaultMutableTreeNode(new HelpInfo("Input panel","/help/inputpanel.html"));
		category.add(topic);
		topic = new DefaultMutableTreeNode(new HelpInfo("Miscellaneous compartments","/help/inputcompartments.html"));
		category.add(topic);
		
		category = new DefaultMutableTreeNode(new HelpInfo("Settings","/help/settings.html"));
		top.add(category);
		topic = new DefaultMutableTreeNode(new HelpInfo("Criteria","/help/settingscriteria.html"));
		category.add(topic);
		topic = new DefaultMutableTreeNode(new HelpInfo("Genetic","/help/settingsgenetic.html"));
		category.add(topic);
		
		category = new DefaultMutableTreeNode(new HelpInfo("Output panel","/help/output.html"));
		top.add(category);
		topic = new DefaultMutableTreeNode(new HelpInfo("General condition","/help/outputgeneral.html"));
		category.add(topic);
		topic = new DefaultMutableTreeNode(new HelpInfo("Stability","/help/outputstability.html"));
		category.add(topic);
		topic = new DefaultMutableTreeNode(new HelpInfo("Strength curves","/help/outputstrength.html"));
		category.add(topic);
		topic = new DefaultMutableTreeNode(new HelpInfo("Genetic","/help/outputgenetic.html"));
		category.add(topic);
	}
}